<?php

class Zend_Pdf_Signature {
    protected $_maxSignatureLength = 25000;
    protected $_byteRangeSegmentLength = 15;
    protected $_tmpDir = './tmp';
    protected $_privKeyCert = '';
    protected $_privKeyPassword = '';
    
    protected $_sigParams = array(
        'Name' => '',
        'Location' => '',
        'Reason' => '',
        'ContactInfo' => ''
    );
    
    // enable timestamped signature
    protected $_enableTSA = false;
    // TSA
    protected $_TSAParams = array(
        'TSA_username' => '',
        'TSA_password' => '',
        'TSA_url'      => '',
        'TSA_port'     => 80,
        'TSA_cert'     => '',
        'TSA_algorithm' => 'sha1'
    );
    
    public function __construct($privKeyCert, $privKeyPassword = '', $tmpDir = 'tmp', $enableTSA = false)
    {
        if(!file_exists($privKeyCert)) {
            throw new Zend_Pdf_Exception('privKeyCert must be a file in PEM format',Zend_Pdf_Exception::BAD_PARAMETER_TYPE);
        }
        $this->_privKeyCert = 'file://'.realpath($privKeyCert);
        $this->_privKeyPassword = $privKeyPassword;
        
        if($enableTSA) {
            $this->_enableTSA = true;
        }
        
        $this->setTmpDir($tmpDir);
    }
    
    public function setTSAEnabled()
    {
        $this->_enableTSA = true;
    }
    
    public function isTSAEnabled()
    {
        return $this->_enableTSA;
    }
    
    public function applyTSA($signature)
    {
        if(empty($signature) || !count($this->_TSAParams)) {
            throw new Zend_Pdf_Exception('TSAParams must be set before calling applyTSA');
        }
        return $this->_timestampRequest($signature);
    }
    
    public function setTSAParams(Array $TSAParams)
    {
        if(!count($TSAParams) || 
           !array_key_exists('TSA_url',$TSAParams)) {
            throw new Zend_Pdf_Exception('TSAParams must be an array with the following keys: "TSA_url"',Zend_Pdf_Exception::BAD_PARAMETER_TYPE);
        }
        
        foreach($TSAParams as $paramName => $paramValue) {
            if(in_array($paramName,array_keys($this->_TSAParams))) {
                $this->_TSAParams[$paramName] = $paramValue;
            }
        }
    }
    
    public function getTSAParam($paramName)
    {
        if(array_key_exists($paramName,$this->_TSAParams)) {
            return $this->_TSAParams[$paramName];
        } else {
            return null;
        }
    }
    
    public function setSigParams(Array $SigParams)
    {
        foreach($SigParams as $paramName => $paramValue) {
            if(in_array($paramName,array_keys($this->_sigParams))) {
                $this->_sigParams[$paramName] = $paramValue;
            }
        }
    }
    
    public function getSigParam($paramName)
    {
        if(array_key_exists($paramName,$this->_sigParams)) {
            return $this->_sigParams[$paramName];
        } else {
            return null;
        }
    }
    
    public function setTmpDir($tmpDir)
    {
        if(($dir = realpath($tmpDir)) !== false) {
            $this->_tmpDir = $dir;
        } else {
            throw new Zend_Pdf_Exception('Temporary directory must be writeable by current user',Zend_Pdf_Exception::BAD_FILE_PATH);
        }
    }
    
    public function getTmpDir()
    {
        return $this->_tmpDir;
    }
    
    public function getMaxSignatureLength()
    {
        return $this->_maxSignatureLength;
    }
    
    public function getByteRangeSegmentLength()
    {
        return $this->_byteRangeSegmentLength;
    }
    
    public function getPrivKeyCert()
    {
        return $this->_privKeyCert;
    }
    
    public function getPrivKeyPassword()
    {
        return $this->_privKeyPassword;
    }
    
    
    protected function _timestampRequest($data)
    {
        $parsedSignature = Zend_Asn1_Base::parse($data);
        $data = $parsedSignature->nodes[1]->nodes[0]->nodes[4]->nodes[0]->nodes[5]->getValue();

        $data = array(
            new Zend_Asn1_Base(Zend_Asn1_Integer::IDENTIFIER, "\x01"),
            // MessageImprint
            new Zend_Asn1_Base(Zend_Asn1_Base::CONSTRUCTED | Zend_Asn1_Sequence::IDENTIFIER, '', array(
                // hashAlgorithm
                new Zend_Asn1_Base(Zend_Asn1_Base::CONSTRUCTED | Zend_Asn1_Sequence::IDENTIFIER, '', array(
                    new Zend_Asn1_Base(Zend_Asn1_ObjectIdentifier::IDENTIFIER, Zend_Asn1_Base::encode('1.3.14.3.2.26')), // SHA1
                    new Zend_Asn1_Base(Zend_Asn1_Null::IDENTIFIER)
                )),
                // hashedMessage
                new Zend_Asn1_Base(Zend_Asn1_OctetString::IDENTIFIER, sha1($data,true))
            )),
            new Zend_Asn1_Base(Zend_Asn1_Integer::IDENTIFIER, sprintf('%c%c%c%c%c%c%c%c', rand(0,256), rand(0,256), rand(0,256), rand(0,256), rand(0,256), rand(0,256), rand(0,256), rand(0,256))),
            new Zend_Asn1_Base(Zend_Asn1_Boolean::IDENTIFIER, "\xFF")
        );

         $request = new Zend_Asn1_Base(Zend_Asn1_Base::CONSTRUCTED | Zend_Asn1_Sequence::IDENTIFIER, '', $data);

        $adapter = new Zend_Http_Client_Adapter_Curl();
        
        $client  = new Zend_Http_Client($this->_TSAParams['TSA_url']);
        if(strpos($this->_TSAParams['TSA_url'],'https') !== false)
        {
            $adapter->setCurlOption(CURLOPT_SSL_VERIFYPEER,0);
            if(!empty($this->_TSAParams['TSA_cert'])) {
                $adapter->setCurlOption(CURLOPT_CAINFO,$this->_TSAParams['TSA_cert']);
            }
        }
        if($this->_TSAParams['TSA_port'] != 80)
        {
            $adapter->setCurlOption(CURLOPT_PORT,$this->_TSAParams['TSA_port']);
        }
        $client->setAdapter($adapter);
        
        $response = $client->setRawData($request->__toString(), 'application/timestamp-query')->request('POST');

        $response = Zend_Asn1_Base::parse($response->getBody());

        $tsToken = $response->nodes[1];
        $tsEntry = new Zend_Asn1_Base(Zend_Asn1_Set::IDENTIFIER | Zend_Asn1_Base::CONSTRUCTED, $tsToken->__toString());

        $parsedSignature->nodes[1]->nodes[0]->nodes[4]->nodes[0]->addNode(
            new Zend_Asn1_Base(Zend_Asn1_Base::CONTEXT_SPECIFIC | Zend_Asn1_Base::CONSTRUCTED | "\x01", '', array(
                new Zend_Asn1_Base(Zend_Asn1_Sequence::IDENTIFIER | Zend_Asn1_Base::CONSTRUCTED, '', array(
                    new Zend_Asn1_Base(Zend_Asn1_ObjectIdentifier::IDENTIFIER, Zend_Asn1_Base::encode('1.2.840.113549.1.9.16.2.14')),
                    $tsEntry
                ))
            ))
        );

        return $parsedSignature->__toString();
    }
}